function dat = AsolveIterFromSpectrum(specdat,tdat,varargin)
%  function dat = AsolveIterFromSpectrum(specdat,tdat,varargin)
%    Solve for A-space line integrals using maximum likelihood from PHA data with no pileup.
%  Approach:
%  compute likelihood from assumed known spectrum incident on object + atten. coeff
%  assume PHA with no pileup. Assume Poisson distribution
%    inputs:
%      specdat: a spectrum struct with fields
%        specnum: photon number spectrum
%        egys: photon energies for specnum
%        idx_threshold: indexes of bin edges into specnum and energies vectors. nbins is numel(idx_threshold)+1
%        mus: attenuation coefficients of basis materials at egys(rows). nbasis is number of columns
%      tdat: a struct with test data with fields
%        ns: a matrix of counts for each bin, one row per reading, same number of columns same as nbins
%        As: the A values for each row of logI
%      init_vals: (optional=[]) matrix, same size as tdat.As  If present uses these initial values for iteration
%    outputs:
%      dat: a struct with fields
%        the input data
%        Astest: the A values for the test data (tdat)
%        errs: the std dev of the errors with tdat
%           See code for other fields
% ToDo: Example of use
%    REA 4/Jul/2013 17:08 - 5/Jul/2013 8:59
nreqargs = 2;
assert(nargin>=nreqargs);
solvedat = [];
init_vals = [];
verbose = false;


if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'verbose';          verbose = varargin{i+1};  assert(isa(verbose,'logical'));   i=i+1;
     case 'solvedat';         solvedat=varargin{i+1};  assert(isstruct(solvedat));   i=i+1;
     case 'init_vals';         init_vals=varargin{i+1};  assert( all(size(init_vals)==size(tdat.As)) );   i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end


options.Display = 'off'; % used on iterative methods


if isempty(solvedat)
      % parameters
    [specdat.negys,specdat.nbasis] = size(specdat.mus);
      % indexes to bins
    nbins = numel(specdat.idx_threshold)+1;       
    bin_edges = [0; specdat.idx_threshold(:);numel(specdat.egys)];
    idxs2ins = cell(nbins,1);
    ns_air = zeros(nbins,1);
    for k = 1:nbins
        idxs2bins{k}= (bin_edges(k)+1):bin_edges(k+1);
        ns_air(k) = sum(specdat.specnum(idxs2bins{k}));
    end
    specdat.nbins = nbins;
    specdat.idxs2bins = idxs2bins;
    specdat.ns_air = ns_air;
    specdat.log_ns_air = log(ns_air);
    solvedat.specdat= specdat; % save in solvedat struct so can be accessed by minimization funcs
      % linear MLE
   M = mubarNbinNdim(specdat,zeros(1,specdat.nbasis)); % use spectrum at 0 thickness
   RLi = diag(LambdasLoc(specdat,zeros(1,specdat.nbasis)));
   solvedat.cs_mle = inv(M'*RLi*M)*M'*RLi;
end
dat.solvedat = solvedat;

if isempty(init_vals)
    dat.As_initfit = tdat.logI*(solvedat.cs_mle');  
else
    dat.As_initfit = init_vals;
end
dat.Astest = zeros(size( dat.As_initfit ) ); % will hold results
n2test = size(dat.Astest,1);
times = zeros(n2test,1);
for kpnt = 1:n2test
   tic
    solvedat.nspnt = tdat.ns(kpnt,:)'; 
     % pass counts in as member of spectrum struct so 
     % spectrum and mus accessible to logProbFuncLoc
    As=fminsearch(@(A) logPoissProbFuncLoc(A,solvedat),dat.As_initfit(kpnt,:),options);
    dat.Astest(kpnt,:) = As';   
    times(kpnt) = toc;
    if verbose 
        if mod(kpnt,100)==0
            fprintf('%d ',floor(kpnt/100)); 
        end
        if mod(kpnt,1000)==0
           fprintf('\n');
        end
    end
end
if verbose,  fprintf('\n'); end
dat.Asdiff = dat.Astest - tdat.As;
dat.times = times;


% -------local functions---------------------
   
function logp = logPoissProbFuncLoc(A,solvedat)
% (negative of) exponent of Poisson pdf for given A. 
% The measurement is counts for each PHA bin, passed in as solvedat.nspnt
    lambdas = LambdasLoc(solvedat.specdat,A);
       % use negative of log likelihood so get maximum from fminsearch
    logp = -sum(solvedat.nspnt.*log(lambdas) -lambdas); 

function lambdas = LambdasLoc(sp,As)
% the integral (number of counts) of the spectrum in each of the PHA bins
    specnum_transmitted = sp.specnum(:).*exp(-sp.mus*As(:));
    lambdas = zeros(sp.nbins,1);
    for kbin = 1:sp.nbins
       lambdas(kbin) = sum(specnum_transmitted(sp.idxs2bins{kbin}));
    end

    